#!/usr/bin/env python
#
#    Copyright (c) 2011-2012 Andrey Sibiryov <me@kobology.ru>
#    Copyright (c) 2011-2012 Other contributors as noted in the AUTHORS file.
#
#    This file is part of Cocaine.
#
#    Cocaine is free software; you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    Cocaine is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program. If not, see <http://www.gnu.org/licenses/>. 
#

import zmq

from optparse import OptionParser
from sys import argv, exit

def main(app, hosts, timeout):
    context = zmq.Context()

    for host in hosts:
        request = context.socket(zmq.REQ)
        request.setsockopt(zmq.LINGER, 0)
        request.connect('tcp://%s:5000' % host)

        # Statistics
        request.send_json({
            'version': 2,
            'action': 'info'
        })

        poller = zmq.Poller()
        poller.register(request, zmq.POLLIN)

        # Poll the socket for response for 3 seconds
        sockets = dict(poller.poll(timeout = timeout))

        if request in sockets and sockets[request] == zmq.POLLIN:
            response = request.recv_json()
            
            if app not in response["apps"]:
                print "The '%s' app was not found on '%s'." % (app, host)
                exit(1)

            if response["apps"][app]["state"] != "running":
               print "The '%s' app is not running on '%s'." % (app, host)
               exit(1)
        else:
            print "Host '%s' is not responding." % host
            exit(1)

description = """This tool allows you to check if the specified app
is up and running on one or multiple nodes. On success you'll get
no output and an exit code of '0', otherwise the error message will
describe the problem, and the exit code will be '1'."""
    
usage = "Usage: %prog <app-name> [options] [<host-name-1> ... <host-name-N>]"

version = "0.9.6"

if __name__ == "__main__":
    parser = OptionParser(usage = usage,
                          description = description,
                          version = version)

    parser.add_option("-t", "--timeout",
                      type = "int",
                      default = 3000,
                      help = "The amount of time to wait for the node to respond, in milliseconds")
    
    (options, args) = parser.parse_args()

    if len(args) == 0:
        parser.print_usage()
    elif len(args) == 1:
        main(args[0], ['localhost'], options.timeout)
    else:
        main(args[0], args[1:], options.timeout)

